-module(gmhc_config).

-export([ load_config/0
        , get_config/1
        , verify_cfg_props/1
        ]).

-include_lib("kernel/include/logger.hrl").

load_config() ->
    instrument_gmconfig(),
    ?LOG_DEBUG("Schema: ~p", [gmconfig:schema()]),
    gmconfig:load_user_config(report),
    gmconfig:apply_os_env(),
    gmconfig:process_plain_args(),
    ok.

instrument_gmconfig() ->
    gmconfig:set_gmconfig_env(gmconfig_env()).

-spec gmconfig_env() -> gmconfig:gmconfig().
gmconfig_env() ->
    #{ os_env_prefix => "GMHC"
     , config_file_basename => "gmhive_client_config"
     , config_file_os_env => "GMHIVE_CLIENT_CONFIG"
     , config_file_search_path => [".", fun setup:home/0, fun setup:data_dir/0 ]
     , config_plain_args => "-gmhc"
     , system_suffix => ""
     , schema => fun gmhc_config_schema:to_json/0 }.

get_config(Path) ->
    gmconfig:get_config(Path, cpath(Path)).

cpath([<<"pubkey">>])                -> [user_config, env(pubkey        )                ];
cpath([<<"extra_pubkeys">>])         -> [user_config, env(extra_pubkeys ), schema_default];
cpath([<<"pool_admin">>, <<"url">>]) -> [user_config, env(pool_admin_url), schema_default];
cpath([<<"workers">>])               -> [user_config, env(workers       ), schema_default];
cpath(_)                             -> [user_config,                      schema_default].

env2cpath(pubkey        ) -> [<<"pubkey">>];
env2cpath(extra_pubkeys ) -> [<<"extra_pubkeys">>];
env2cpath(pool_admin_url) -> [<<"pool_admin">>, <<"url">>];
env2cpath(workers       ) -> [<<"workers">>].

verify_cfg_props(PropList) ->
    Cfg = lists:foldl(
            fun({K,V}, M) ->
                    CfgK = env2cpath(K),
                    gmconfig:merge_config_maps(M, to_cfg_map(CfgK, V))
            end, #{}, PropList),
    case gmconfig_schema_utils:valid(Cfg) of
        Ok when is_map(Ok) ->
            ok;
        Other ->
            error({invalid_config, Other})
    end.

to_cfg_map(K, V) ->
    to_cfg_map(K, V, #{}).

to_cfg_map([H], V, M) ->
    M#{H => V};
to_cfg_map([H|T], V, M) ->
    M#{H => to_cfg_map(T, V, M)}.

env(K) ->
    {env, gmhive_client, K}.
